package se.pmws1213.diaballik.controller;

import java.beans.PropertyChangeEvent;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

import se.pmws1213.DiaballikGame;
import se.pmws1213.Field;
import se.pmws1213.Move;
import se.pmws1213.Pass;
import se.pmws1213.Player;
import se.pmws1213.diaballik.Util;
import se.pmws1213.diaballik.gui.DiaballikMainScreen;
import se.pmws1213.diaballik.gui.DiaballikStartScreen;

public class GameController extends AbstractController
{
	private DiaballikMainScreen mainScreen;
	private DiaballikGame game;
	
	public GameController(DiaballikGame game, DiaballikMainScreen diaballikMainScreen)
	{
		this.mainScreen = diaballikMainScreen;
		this.game = game;
	}

	public void start()
	{
		Display display = Display.getDefault();
		mainScreen.createContents();
		
		for(Field field : game.getFields())
		{
			FieldController fController = new FieldController(field, Util.findLabelForField(mainScreen, field), mainScreen);
			fController.refreshUI();
			fController.start();
		}
		
		Label instructionLabel = mainScreen.getLblInstruction();
		
		for(Player player : game.getPlayers())
		{
			PlayerController pController = new PlayerController(mainScreen, player, instructionLabel);
			pController.start();
		}
		
		Util.centerShell(mainScreen.getGameShell());
		mainScreen.getGameShell().open();
		mainScreen.getGameShell().layout();
		mainScreen.getGameShell().pack();
		
		mainScreen.getBtnSkip().setEnabled(false);
		
		super.start();
		
		refreshUI();
		
		while (!mainScreen.getGameShell().isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
	}
	
	public void stop()
	{
		super.stop();
		mainScreen.getGameShell().dispose();
		
		//new game
		LoginController startGameController = new LoginController(
				new DiaballikGame(), new DiaballikStartScreen());
		startGameController.start();
	}
	
	@Override
	public void propertyChange(PropertyChangeEvent arg0)
	{
		refreshUI();
	}

	@Override
	public void refreshUI()
	{
		Player currentPlayer = game.getCurrentPlayer();
		if (currentPlayer != null)
		{
			mainScreen.getLblCurrentPlayerName().setText(currentPlayer.getName());
			
			Color color = null;
			if(currentPlayer.getColor().equals("Red"))
			{
				color = new Color(null, 255, 0, 0);
			}
			else
			{
				color = new Color(null, 0, 255, 0);
			}
			mainScreen.getLblCurrentPlayerName().setBackground(color);
		}
		
		//winner
		Player winner = game.getWinner();
	    if (winner != null)
	    {
	    	MessageDialog.openInformation(mainScreen.getGameShell(), "Winner", "Player " + winner.getName() + " has won the game!");
	    	stop();
	    }
	}

	@Override
	public void registerListeners()
	{
		mainScreen.getBtnSkip().addSelectionListener(new SelectionListener() {
			@Override
			public void widgetSelected(SelectionEvent arg0)
			{
				//nextPlayer
				game.getCurrentPlayer().removeAllFromActions();
				game.setCurrentPlayer(game.getCurrentPlayer().getNext());
				game.getCurrentPlayer().withActions(new Move()).withActions(new Move()).withActions(new Pass());
				mainScreen.getBtnSkip().setEnabled(false);
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0){}
		});
		
		listenTo(DiaballikGame.PROPERTY_WINNER, game);
		listenTo(DiaballikGame.PROPERTY_CURRENT_PLAYER, game);
	}
}
