package se.pmws1213.diaballik.controller;

import java.beans.PropertyChangeEvent;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Display;

import se.pmws1213.DiaballikGame;
import se.pmws1213.diaballik.Util;
import se.pmws1213.diaballik.gui.DiaballikMainScreen;
import se.pmws1213.diaballik.gui.DiaballikStartScreen;

public class LoginController extends AbstractController
{
	private DiaballikStartScreen startScreen;
	private DiaballikGame game;

	public LoginController(DiaballikGame diaballikGame, DiaballikStartScreen diaballikStartScreen)
	{
		this.startScreen = diaballikStartScreen;
		this.game = diaballikGame;
	}

	public void start()
	{
		Display display = Display.getDefault();
		startScreen.createContents();
		Util.centerShell(startScreen.getStartGameShell());
		startScreen.getStartGameShell().open();
		startScreen.getStartGameShell().layout();
		
		super.start();
		
		while (!startScreen.getStartGameShell().isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
	}

	public void stop()
	{
		super.stop();
		startScreen.getStartGameShell().dispose();
	}

	@Override
	public void propertyChange(PropertyChangeEvent arg0){}

	@Override
	public void refreshUI(){}

	@Override
	public void registerListeners()
	{
		startScreen.getStartButton().addSelectionListener(new SelectionListener(){

			@Override
			public void widgetSelected(SelectionEvent arg0)
			{
				String playerName1 = startScreen.getPlayer1Text().getText();
				String playerName2 = startScreen.getPlayer2Text().getText();


				if(!areNamesValid(playerName1, playerName2))
				{
					MessageDialog.openError(startScreen.getStartGameShell(), "Error", "Enter two different usernames!");
				}
				else
				{
					game.init(playerName1, playerName2);

					GameController gameController = new GameController(game, new DiaballikMainScreen());
					LoginController.this.stop();
					gameController.start();
				}
			}

			//check if usernames are valid, valid means different from "Player.." and no doubles
			private boolean areNamesValid(final String playerName1, final String playerName2)
			{          
				if(!playerName1.isEmpty() && !playerName2.isEmpty() &&
						!playerName1.matches("Player 1") && !playerName2.matches("Player 2"))
				{
					if(!playerName1.equals(playerName2)) // no double user names
					{
						return true;
					}
				}
				return false;
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		startScreen.getExitButton().addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0)
			{
				LoginController.this.stop();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
	}
}
